package gui;

import sudoku.SudokuSolver;
import java.awt.event.*;

/**
 * SolverController is a controller for the SudokuSolver interface
 */
public class SudokuController {
    SudokuSolver model;
    SudokuView view;

    /**
     * Constructor
     * 
     * @param model SudokuSolver model
     * @param view  SudokuView view
     */
    public SudokuController(SudokuSolver model, SudokuView view) {
        this.model = model;
        this.view = view;

        // Add action listeners to the buttons
        view.addSolveButtonListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                // Solve the board
                model.solve();

                // Update the view
                view.updateView(model.getBoard());
            }
        });

        view.addResetButtonListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                // Clear the board
                model.clear();

                // Update the view
                view.updateView(model.getBoard());
            }
        });

        view.addRandomButtonListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                // Randomize the board
                model.randomizeBoard();

                // Update the view
                view.updateView(model.getBoard());
            }
        });

        view.addCellClickListener(new CellActionListener());
    }

    /** Start the GUI */
    public void start() {
        view.setVisible(true);
    }

    /**
     * CellActionListener is an ActionListener for the Sudoku grid cells
     */
    private class CellActionListener implements ActionListener {
        @Override
        public void actionPerformed(ActionEvent e) {
            // Get the row and column from the clicked cell
            int row = view.getSelectedRow();
            int col = view.getSelectedColumn();

            String inputText = view.getCellValue(row, col);
            int value = Integer.parseInt(inputText);
            // Check if the input is legal and update the model and view
            if (model.isLegal(row, col, value)) {
                model.set(row, col, value);
                view.updateView(model.getBoard());
            } else {
                view.showErrorMessage("Invalid input. Try again.");
            }
        }
    }
}
